// A very simple XOR drawing demonstrator
//
// (c) Petr Felkel, 2008
// Fundamentals of computer graphics (Zaklady pocitacove grafiky)
// ////////////////////////////////////////////////

#include <stdlib.h>
#include <GL/glut.h>

static int x = 120;
static int y = 140;
static int xp, yp;   // previous coords to XOR out
static int xStart=20;
static int yStart=20;
static bool moving =  false;
static int winHeight;
static int winWidth;
static GLuint texName;
static float lineWidth = 1.0f;


#define textureHeight 128
#define textureWidth 128
void generateTexture()
{
	GLubyte randomTexture[textureHeight][textureWidth][4];
	
	for(int i = 0; i < textureHeight; i++ )
		for( int j = 0; j < textureWidth; j++ )
		{
			randomTexture[i][j][0] = (double)rand() / (RAND_MAX + 1) * 255;
			randomTexture[i][j][1] = (double)rand() / (RAND_MAX + 1) * 255;
			randomTexture[i][j][2] = (double)rand() / (RAND_MAX + 1) * 255;
			randomTexture[i][j][3] = 255;
		}

	glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, textureWidth, textureHeight, 
		  0, GL_RGBA, GL_UNSIGNED_BYTE, randomTexture );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexEnvi( GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE );
}
void drawBackground()
{
	glEnable( GL_TEXTURE_2D );

	glBegin( GL_TRIANGLES );
	    glTexCoord2d(0.0, 0.0); glVertex2i( 0, 0 );
		glTexCoord2d(1.0, 1.0); glVertex2i( winWidth - 1 , winHeight - 1 );
		glTexCoord2d(0.0, 1.0); glVertex2i( 0, winHeight - 1 );
	glEnd();
	glDisable( GL_TEXTURE_2D );
}

void myRecti (int x1, int y1, int x2, int y2)
{
	glBegin( GL_LINE_LOOP );
	  glVertex2i( x1, y1 );
	  glVertex2i( x2, y1 );
	  glVertex2i( x2, y2 );
	  glVertex2i( x1, y2 );
	glEnd();
}
void mouse( int button, int state, int x_, int y_)
{
	if( button == GLUT_LEFT_BUTTON )
		if( state == GLUT_DOWN ) 
		{
			glClear( GL_COLOR_BUFFER_BIT );
			drawBackground();
			
			moving = true;
			xStart = x = xp = x_;
			yStart = y = yp = winHeight - y_ - 1;
		}
		else
		{
			moving = false;
		}

		glutPostRedisplay();
}
void mouseMove( int x_, int y_)
{
	xp = x;  x = x_;
	yp = y;  y = winHeight - y_ - 1;

	glutPostRedisplay();

}
void keyboard( unsigned char key, int x, int y)
{
	switch( key )
	{
	case 27:
	case 'q':
		exit(0);
	case ' ':
		lineWidth += 3.0;
		if( lineWidth > 15 )
			lineWidth = 1.0f;

		glLineWidth( lineWidth );
		break;
	case 'S':
		glEnable( GL_LINE_SMOOTH );
		glHint( GL_LINE_SMOOTH_HINT, GL_NICEST);
		break;
	case 's':
		glDisable( GL_LINE_SMOOTH );
		break;

	}

	glutPostRedisplay();
}
void reshape( int w, int h )
{
	glViewport( 0,0,w,h);
	winHeight = h;
	winWidth = w;
	glLoadIdentity();
	glOrtho( 0,w, 0, h, -1.0, 10.0);
	glutPostRedisplay();
}


void display(void)
{
	if( moving )
	{
		glEnable( GL_COLOR_LOGIC_OP );
		glLogicOp( GL_XOR );
		
		myRecti( xStart, yStart, xp, yp );       // XOR the previous
		myRecti( xStart, yStart, x, y );         // XOR the current

		glDisable( GL_COLOR_LOGIC_OP );

	}
	else
	{

		glClear( GL_COLOR_BUFFER_BIT );
		drawBackground();
		myRecti( xStart, yStart, x, y );

	}
	glFlush();
}
void main(int argc, char**argv)
{
	glutCreateWindow("XOR interaction");

	glutDisplayFunc( display );
	glutReshapeFunc( reshape );
	glutKeyboardFunc( keyboard );
	glutMouseFunc( mouse );
	glutMotionFunc( mouseMove );

	generateTexture();

	glutMainLoop();
}